<?php

/**
 *
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\Newsletter\Controller\Subscriber;

use Magento\Customer\Api\AccountManagementInterface as CustomerAccountManagement;
use Magento\Customer\Model\Session;
use Magento\Customer\Model\Url as CustomerUrl;
use Magento\Framework\App\Action\Context;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Newsletter\Model\SubscriberFactory;

class EditLandingAction extends \Magento\Framework\App\Action\Action {

    /**
     * Sender email config path
     */
    const XML_PATH_EMAIL_SENDER = 'filoblu_newsletter_section/newsletter_edit_email/sender_email_identity';

    /**
     * Email template config path
     */
    const XML_PATH_EMAIL_TEMPLATE = 'filoblu_newsletter_section/newsletter_edit_email/email_template';

    protected $jsonResultFactory;
    public $_helper;
    protected $_modelThankYou;
    protected $_subscriberInfoFactory;
    protected $_subscriberFactory;
    protected $_storeManager;

    /**
     * @var \Magento\Framework\Mail\Template\TransportBuilder
     */
    protected $_transportBuilder;

    /**
     * @var \Magento\Framework\Translate\Inline\StateInterface
     */
    protected $inlineTranslation;

    public function __construct(
    Context $context, \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory, \FiloBlu\Newsletter\Helper\Data $helper, \FiloBlu\Newsletter\Model\ThankYou $modelThankYou, \FiloBlu\Newsletter\Model\SubscriberInfoFactory $subscriberInfoFactory, \Magento\Newsletter\Model\SubscriberFactory $subscriberFactory, \Magento\Store\Model\StoreManagerInterface $storeManager, \FiloBlu\WorkWithUs\Model\Mail\TransportBuilder $transportBuilder, \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation
    ) {
        $this->resultJsonFactory = $resultJsonFactory;
        $this->_helper = $helper;
        $this->_modelThankYou = $modelThankYou;
        $this->_subscriberInfoFactory = $subscriberInfoFactory;
        $this->_subscriberFactory = $subscriberFactory;
        $this->_storeManager = $storeManager;
        $this->_transportBuilder = $transportBuilder;
        $this->inlineTranslation = $inlineTranslation;

        parent::__construct($context);
    }

    /**
     * New subscription action
     *
     * @throws \Magento\Framework\Exception\LocalizedException
     * @return void
     */
    public function execute() {

        if ($this->_helper->getConfig("general/enabled") && $this->getRequest()->isPost() && $subscriberId = $this->getRequest()->getPost('subscriber_id')) {

            $resultJson = $this->resultJsonFactory->create();

            try {

                $model = $this->_subscriberInfoFactory->create()->load($subscriberId);

                if (!$model->getId()) {
                    throw new \Exception("Subscriber not found");
                } else {

                    $subscriber = $this->_subscriberFactory->create()->load($subscriberId);

                    // Add subscriber info data to subscriber model
                    foreach ($subscriber->getData() as $key => $value) {
                        $model->setData($key, $value);
                    }

                    $fieldsMappingGetCompare = unserialize($this->_helper->getConfig('general/fields_get_compare_newsletter_edit'));

                    if ($fieldsMappingGetCompare && count($fieldsMappingGetCompare)) {
                        foreach ($fieldsMappingGetCompare as $field) {
                            if ($this->getRequest()->getParam($field["getname"])) {
                                if ($model->getData($field["dbcolumnname"]) != $this->getRequest()->getParam($field["getname"])) {
                                    throw new \Exception("Hash not correct");
                                }
                            } else {
                                throw new \Exception("Hash not found");
                            }
                        }
                    }
                }

                $fieldsMapping = unserialize($this->_helper->getConfig('general/fields_mapping'));

                if ($fieldsMapping && count($fieldsMapping)) {
                    $post = $this->getRequest()->getPost();

                    foreach ($fieldsMapping as $field) {
                        if (isset($post[$field["postname"]])) {
                            if (is_array($post[$field["postname"]])) {
                                $valStr = "";
                                foreach ($post[$field["postname"]] as $val) {
                                    if (is_string($val)) {
                                        if ($valStr) {
                                            $valStr .= ' | ' . $val;
                                        } else {
                                            $valStr = $val;
                                        }
                                    }
                                }

                                if ($valStr) {
                                    $model->setData($field["dbcolumnname"], $valStr);
                                }
                            } else {
                                $model->setData($field["dbcolumnname"], $post[$field["postname"]]);
                            }
                        }
                    }

                    $model->save();
                }

                // Send email 
                if ($this->_helper->getConfig('newsletter_edit_email/enabled')) {
                    $this->inlineTranslation->suspend();

                    $postObject = new \Magento\Framework\DataObject();

                    $this->_transportBuilder
                            ->setTemplateIdentifier($this->_helper->getConfig('newsletter_edit_email/email_template'))
                            ->setTemplateOptions(
                                    [
                                        'area' => \Magento\Backend\App\Area\FrontNameResolver::AREA_CODE,
                                        'store' => \Magento\Store\Model\Store::DEFAULT_STORE_ID,
                                    ]
                            )
                            ->setTemplateVars(['data' => $postObject])
                            ->setFrom($this->_helper->getConfig('newsletter_edit_email/sender_email_identity'))
                            ->addTo($model->getSubscriberEmail(), $model->getFirstname() . " " . $model->getLastname());

                    $transport = $this->_transportBuilder->getTransport();

                    $transport->sendMessage();

                    $this->inlineTranslation->resume();
                }


                if ($this->getRequest()->getPost('thankyou_id') && $this->getRequest()->getPost('thankyou_id') != 0) {
                    $this->_helper->setThankYouCookie($this->getRequest()->getPost('thankyou_id'));
                    $resultJson->setData(['success' => true, 'error' => false, 'redirect' => $this->_storeManager->getStore()->getBaseUrl() . $this->_modelThankYou->getUrlThankYouPageById($this->getRequest()->getPost('thankyou_id'))->getData()["url_key"]]);
                } else {
                    $resultJson->setData(['success' => true, 'error' => false, 'message' => __("Thank you for updated data."), 'redirect' => $this->_storeManager->getStore()->getBaseUrl()]);
                }

                //  Send mail
            } catch (\Magento\Framework\Exception\LocalizedException $e) {
                $resultJson->setData(['success' => false, 'error' => true, 'message' => __('There was a problem with updated data: %1', $e->getMessage())]);
            } catch (\Exception $e) {
                $resultJson->setData(['success' => false, 'error' => true, 'message' => __($e->getMessage())]);
            }

            return $resultJson;
        } else {
            return null;
        }
    }

}
